#include <iostream>
#include <fstream>
#include <string>
#include "maze.h"
using namespace std;

Maze::Maze()
:maze(0),cur_row(0),cur_col(0),num_rows(0),num_cols(0)
{
}

Maze::Maze(const char *filename)
{
    if (!read(filename)) {
        maze=0;
        cur_row=0;
        cur_col=0;
        num_rows=0;
        num_cols=0;
    }
}

Maze::Maze(const Maze &copy)
:maze(0), cur_row(copy.cur_row), cur_col(copy.cur_col), num_rows(copy.num_rows),
num_cols(copy.num_cols)
{
    cout << "copy constructor called for Maze class " << endl;
    int i;
    if (num_rows*num_cols) {
        maze = new char[num_rows*num_cols];
        for (i = 0; i < num_rows*num_cols; i++) 
           maze[i] = copy.maze[i];
    }
}

Maze::~Maze()
{
    cout << "destructor called for maze class" << endl;
    if (maze) delete [] maze;
    maze=0;
    cur_row=0;
    cur_col=0;
    num_rows=0;
    num_cols=0;
}

// read specific format for maze file, stored in file filename
// weak on error checking and recovery
bool Maze::read(const char *filename)
{
    ifstream infile;
    string trash;
    int i, j;
    infile.open(filename, ios::in);
    if (!infile) return false;
    for (i=0; i<2; i++) infile >> trash;     // ignore the first two words
    infile >> num_rows; // read the number of rows in the maze
    infile >> trash;    // skip another word
    infile >> num_cols; // read the number of columns in the maze
    for (i=0; i < 5; i++) infile >> trash;  // skip 5 words
    infile >> cur_row;                      // read the entrance row
    for (i=0; i<2; i++) infile >> trash;    // skip two words
    infile >> cur_col;                      // read the entrance column
    maze = new char[num_rows*num_cols];     // allocate space for the maze
    for (i = 0; i < num_rows; i++)  
        for (j = 0; j < num_cols; j++)
            infile >> maze[i*num_cols+j];   // read in the maze itself
    return true;
}

// display maze to output
void Maze::show(ostream &output) const
{
    int i, j;
    for (i=0; i < num_rows; i++) {
        for (j=0; j < num_cols; j++)
            cout << maze[i*num_cols+j];
        cout << endl;
    }
}

ostream & operator<<(ostream &output, const Maze &mz)
{
  mz.show(output);
}

const Maze & Maze::operator=(const Maze &rhs)
{
    cout << "assignment operator called for maze class" << endl;
    int i;
    if (this == &rhs) return *this;
    if (maze) delete[]maze;
    maze = 0;
    cur_row = rhs.cur_row;
    cur_col = rhs.cur_col;
    num_rows = rhs.num_rows;
    num_cols = rhs.num_cols;
    if (num_rows*num_cols) {
        maze = new char[num_rows*num_cols];
        for (i = 0; i < num_rows*num_cols; i++) 
           maze[i] = rhs.maze[i];
    }
    return *this;
}

bool Maze::step(ostream &output)
{
    cout << "Maze: function step not implemented" << endl;
    return false;
}

void Maze::traverse(ostream &output)
{
    cout << "Maze: function traverse not implemented" << endl;
}

void Maze::generate_new()
{
    cout << "Maze: function generate_new not implemented" << endl;
}
